{ -------------------------------------------------------------------------------------}
{ A "Registry" component for Delphi32.                                                 }
{ Copyright 1996, Jean-Fabien Connault.  All Rights Reserved.                          }
{ This component can be freely used and distributed in commercial and private          }
{ environments, provided this notice is not modified in any way.                       }
{ -------------------------------------------------------------------------------------}
{ Feel free to contact us if you have any questions, comments or suggestions at        }
{ JFConnault@mail.dotcom.fr (Jean-Fabien Connault)                                     }
{ -------------------------------------------------------------------------------------}
{ Date last modified:  08/25/96                                                        }
{ -------------------------------------------------------------------------------------}

{ -------------------------------------------------------------------------------------}
{ TRegistryEnhanced v1.02                                                              }
{ -------------------------------------------------------------------------------------}
{ Description:                                                                         }
{   A component that allows you manipulate entries in the Registry                     }
{ Properties:                                                                          }
{   property EntriesFound: TStringList;      "StringList to store the search result"   }
{   property Key: String;                    "Key in Registry"                         }
{   property RootKey: TRootKey;              "RootKey"                                 }
{ Procedures and Functions:                                                            }
{  procedure Search(ASubKey,AString:string;ASearchOption:TSearchOption);               }
{  procedure SetValue(AValue:string;AValueData:Variant);                               }
{  function GetValue(AValue:String): Variant;                                          }
{  function GetValueDefault(AValue:string;AValueDataDefault:Variant): Variant;         }
{  procedure GetValueNames(ASubKey: String; AList:TStrings);                           }
{  procedure GetSubKeys(ASubKey: String; AList:TStrings);                              }
{  procedure DeleteKey(AKey:String);                                                   }
{  procedure DeleteValue(AValue:String);                                               }
{  procedure RenameKey(OldKey,NewKey:String);                                          }
{  procedure RenameValue(OldValue,NewValue:String);                                    }
{ Needs:                                                                               }
{   TBrkApart component from Patrick Brisacier and Jean-Fabien Connault                }
{                                                                                      }
{ See example contained in example.zip file for more details.                          }
{ -------------------------------------------------------------------------------------}
{ Revision History:                                                                    }
{ 1.00:  + Initial release                                                             }
{ 1.01:  + Bug fixed                                                                   }
{ 1.02:  + Added Registry search capabilities                                          }
{ -------------------------------------------------------------------------------------}

unit RegistryEnhanced;

interface

uses
  SysUtils, Windows, Classes, StdCtrls, Forms, Registry, Dialogs, BrkApart, Controls;

type
 TRootKey = (hkClassesRoot, hkCurrentConfig, hkCurrentUser, hkDynData,hkLocalMachine,hkUsers);
 TSearchOption = (opValue,opData,opValueData);

 TRegistryEnhanced = class(TComponent)
  private
    FKey: String;
    FRootKey: TRootKey;
    FBrkApart: TBrkApart;
    FEntriesFound: TStringList;
    function CheckKey(AKey: String):string;
    function ExtractKeyPath(APath: String): string;
    function ExtractKeyValue(APath: String): string;
    function RetrieveValueNames(AString:string;ASearchOption:TSearchOption):boolean;
  protected
  public
    constructor Create(AOwner: TComponent); Override;
    destructor Destroy; Override;
    procedure Search(ASubKey,AString:string;ASearchOption:TSearchOption);
    procedure SetValue(AValue:string;AValueData:Variant);
    function GetValue(AValue:String): Variant;
    function GetValueDefault(AValue:string;AValueDataDefault:Variant): Variant;
    procedure GetValueNames(ASubKey: String; AList:TStrings);
    procedure GetSubKeys(ASubKey: String; AList:TStrings);
    procedure DeleteKey(AKey:String);
    procedure DeleteValue(AValue:String);
    procedure RenameKey(OldKey,NewKey:String);
    procedure RenameValue(OldValue,NewValue:String);
  published
    property EntriesFound: TStringList read FEntriesFound write FEntriesFound;
    property Key: String read FKey write FKey;
    property RootKey: TRootKey read FRootKey write FRootKey default hkCurrentUser;
  end;


procedure Register;

const
 MAX_DEPTH = 100;
 RootKeyValues: array[TRootKey] of integer =
     (HKey_Classes_Root, HKey_Current_Config, HKey_Current_User, HKey_Dyn_Data,HKey_Local_Machine, HKey_Users);

 { French Messages }
 MSG_TOO_MUCH_ENTRIES = 'Trop d''entres.';

 { English Messages }
 {MSG_TOO_MUCH_ENTRIES = 'Too much entries.';}

implementation


{***************************************************************************}
{ CONSTRUCTOR Create                                                        }
{***************************************************************************}

constructor TRegistryEnhanced.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FRootKey := hkCurrentUser;
  FBrkApart := TBrkApart.Create(Self);
  FBrkApart.BreakString := '\';
  FEntriesFound := TStringList.create;
end;

{***************************************************************************}
{ DESTRUCTOR Destroy                                                        }
{***************************************************************************}

destructor TRegistryEnhanced.Destroy;
begin
  FBrkApart.free;
  FEntriesFound.free;
  inherited Destroy;
end;

{***************************************************************************}
{ FUNCTION CheckKey                                                         }
{***************************************************************************}

function TRegistryEnhanced.CheckKey(AKey: string):string;
begin
 // Remove '\' at the end of the key
 if AKey[length(AKey)] = '\' then
    AKey := copy(AKey,1,length(AKey)-1);

 // Remove '\' at the start of the key
 if length(AKey) > 1 then
  if AKey[1] = '\' then
     Akey := copy(AKey,2,length(AKey)-1);

 result := AKey;
 
end;

{***************************************************************************}
{ FUNCTION ExtractKeyPath                                                   }
{***************************************************************************}
function TRegistryEnhanced.ExtractKeyPath(APath: String): string;
var
   i: integer;
begin
  result := '';
  FBrkApart.BaseString := APath;
  FBrkApart.BreakString := '\';
  FBrkApart.AllowEmptyString := false;
  FBrkApart.BreakApart;

  if FBrkApart.StringList.count > 1 then
   begin
    for i := 0 to FBrkApart.StringList.count - 2 do
       result := result + '\' + FBrkApart.StringList[i];
   end;

  // The function returns a path with '\' at start

end;

{***************************************************************************}
{ FUNCTION ExtractKeyValue                                                  }
{***************************************************************************}
function TRegistryEnhanced.ExtractKeyValue(APath: String): string;
begin
  FBrkApart.BaseString := APath;
  FBrkApart.BreakString := '\';
  FBrkApart.AllowEmptyString := false;
  FBrkApart.BreakApart;

  if FBrkApart.StringList.count > 1 then
     result := FBrkApart.StringList[FBrkApart.StringList.count - 1]
  else
     result := APath;

end;

{***************************************************************************}
{ PROCEDURE SetValue                                                        }
{***************************************************************************}

procedure TRegistryEnhanced.SetValue(AValue: string;AValueData:Variant);
var
  Registry: TRegistry;
begin
  if FKey <> '' then
   begin
    Registry := TRegistry.Create;
    try
     Registry.RootKey := RootKeyValues[FRootKey];
     if Registry.OpenKey('\' + CheckKey(FKey) + ExtractKeyPath(AValue),True) then
      begin
       case VarType(AValueData) of
        varString: Registry.WriteString(ExtractKeyValue(AValue),AValueData);
        varInteger: Registry.WriteInteger(ExtractKeyValue(AValue),AValueData);
        varBoolean: Registry.WriteBool(ExtractKeyValue(AValue),AValueData);
        varDate: Registry.WriteDateTime(ExtractKeyValue(AValue),AValueData);
        varCurrency: Registry.WriteCurrency(ExtractKeyValue(AValue),AValueData);
       end;
      end;
     finally
      Registry.Free;
    end;
   end;
end;

{***************************************************************************}
{ FUNCTION GetValue                                                         }
{***************************************************************************}

function TRegistryEnhanced.GetValue(AValue:String): Variant;
var
  Registry: TRegistry;
  AValueData: Variant;
begin
  if FKey <> '' then
   begin
    Registry := TRegistry.Create;
    try
     Registry.RootKey := RootKeyValues[FRootKey];
     if Registry.OpenKey('\' + CheckKey(FKey) + ExtractKeyPath(AValue),False) then
      begin
       case Registry.GetDataType(ExtractKeyValue(AValue)) of
        rdString: AValueData := Registry.ReadString(ExtractKeyValue(AValue));
        rdInteger: AValueData := Registry.ReadInteger(ExtractKeyValue(AValue));
        //rdBinary: ShowMessage('rdBinary');
        //rdExpandString: ShowMessage('rdExpandString');
        //rdUnknown: ShowMessage('rdUnknown');
       else
        //ShowMessage('Aucun type de donnes reconnu.');
       end;
       result := AValueData;
      end;
   finally
      Registry.Free;
    end;
   end;
end;

{***************************************************************************}
{ PROCEDURE GetValueDefault                                                 }
{***************************************************************************}

function TRegistryEnhanced.GetValueDefault(AValue:string;AValueDataDefault:Variant): Variant;
var
  Registry: TRegistry;
  AValueData: Variant;
begin
  if FKey <> '' then
   begin
    Registry := TRegistry.Create;
    try
     Registry.RootKey := RootKeyValues[FRootKey];
     if Registry.OpenKey('\' + CheckKey(FKey) + ExtractKeyPath(AValue),False) then
      begin
        case Registry.GetDataType(ExtractKeyValue(AValue)) of
        rdString:
         try
          AValueData := Registry.ReadString(ExtractKeyValue(AValue));
         except
          AValueData := AValueDataDefault;
         end;
        rdInteger:
         try
          AValueData := Registry.ReadInteger(ExtractKeyValue(AValue));
         except
          AValueData := AValueDataDefault;
         end;
         //rdBinary: ShowMessage('rdBinary');
         //rdExpandString: ShowMessage('rdExpandString');
         //rdUnknown: ShowMessage('rdUnknown');
        else
         //ShowMessage('Aucun type de donnes reconnu.');
         AValueData := AValueDataDefault;
        end;
       result := AValueData;
      end;
    finally
      Registry.Free;
    end;
   end;
end;


{***************************************************************************}
{ PROCEDURE GetValueNames                                                   }
{***************************************************************************}

procedure TRegistryEnhanced.GetValueNames(ASubKey: String; AList:TStrings);
var
  Registry: TRegistry;
  AKey: string;
begin
  if FKey <> '' then
   begin
    Registry := TRegistry.Create;
    try
     Registry.RootKey := RootKeyValues[FRootKey];
     AKey := '\' + CheckKey(FKey);
     if ASubKey <> '' then AKey := AKey + '\' + CheckKey(ASubKey);
     if Registry.OpenKey(AKey,False) then
        Registry.GetValueNames(AList);
    finally
      Registry.Free;
    end;
   end;
end;

{***************************************************************************}
{ PROCEDURE GetSubKeys                                                      }
{***************************************************************************}

procedure TRegistryEnhanced.GetSubKeys(ASubKey: String; AList:TStrings);
var
  Registry: TRegistry;
  AKey: string;
begin
  if FKey <> '' then
   begin
    Registry := TRegistry.Create;
    try
     Registry.RootKey := RootKeyValues[FRootKey];
     AKey := '\' + CheckKey(FKey);
     if ASubKey <> '' then AKey := AKey + '\' + CheckKey(ASubKey);
     if Registry.OpenKey(AKey,False) then
        Registry.GetKeyNames(AList);
    finally
      Registry.Free;
    end;
   end;
end;


{***************************************************************************}
{ PROCEDURE DeleteKey                                                       }
{***************************************************************************}

procedure TRegistryEnhanced.DeleteKey(AKey:String);
var
  Registry: TRegistry;
begin
  if FKey <> '' then
   begin
    Registry := TRegistry.Create;
    try
     Registry.RootKey := RootKeyValues[FRootKey];
     if Registry.OpenKey('\' + FKey,False) then
        Registry.DeleteKey(AKey);
    finally
      Registry.Free;
    end;
   end;
end;

{***************************************************************************}
{ PROCEDURE DeleteValue                                                     }
{***************************************************************************}

procedure TRegistryEnhanced.DeleteValue(AValue:String);
var
  Registry: TRegistry;
begin
  if FKey <> '' then
   begin
    Registry := TRegistry.Create;
    try
     Registry.RootKey := RootKeyValues[FRootKey];
     if Registry.OpenKey('\' + FKey + extractKeyPath(AValue),False) then
        Registry.DeleteValue(extractKeyValue(AValue));
    finally
      Registry.Free;
    end;
   end;
end;


{***************************************************************************}
{ PROCEDURE RenameKey                                                       }
{***************************************************************************}

procedure TRegistryEnhanced.RenameKey(OldKey,NewKey:String);
var
  Registry: TRegistry;
begin
  if FKey <> '' then
   begin
    Registry := TRegistry.Create;
    try
     Registry.RootKey := RootKeyValues[FRootKey];
     if Registry.OpenKey('\' + FKey,False) then
        Registry.RenameValue(OldKey,NewKey);
    finally
      Registry.Free;
    end;
   end;
end;

{***************************************************************************}
{ PROCEDURE RenameValue                                                     }
{***************************************************************************}

procedure TRegistryEnhanced.RenameValue(OldValue,NewValue:String);
var
  Registry: TRegistry;
begin
  if FKey <> '' then
   begin
    Registry := TRegistry.Create;
    try
     Registry.RootKey := RootKeyValues[FRootKey];
     if Registry.OpenKey('\' + FKey + extractKeyPath(OldValue),False) then
        Registry.RenameValue(extractKeyValue(OldValue),NewValue);
    finally
      Registry.Free;
    end;
   end;
end;


{***************************************************************************}
{ FUNCTION RetrieveValueNames                                               }
{***************************************************************************}

function TRegistryEnhanced.RetrieveValueNames(AString:string;ASearchOption:TSearchOption):boolean;
var
   i: integer;
   tslValue: TStringList;
   flag: boolean;
begin
 tslValue := TStringList.create;
 Result := true;
 GetValueNames('',tslValue);

 for i := 0 to tslValue.count - 1 do
  begin
   if AString = '' then flag := true else flag := false;
   case ASearchOption of
    opValue:
      if pos(lowercase(AString),lowerCase(FKey)) > 0 then flag := true;
    opData:
      if pos(lowercase(AString),lowercase(GetValue(tslValue[i]))) > 0 then flag := true;
    opValueData:
      if (pos(lowercase(AString),lowerCase(FKey)) > 0) or
         (pos(lowercase(AString),lowercase(GetValue(tslValue[i]))) > 0) then flag := true;
   end;

   try
    if flag then  EntriesFound.add(FKey + '\' + tslValue[i]);
   except
    on EOutOfResources do
     begin
      Result := False;
      abort;
     end; 
   end;

  end;

 tslValue.free;

end;

{***************************************************************************}
{ PROCEDURE Search                                                          }
{***************************************************************************}
{ ASubKey: SubKey of research                                               }
{ AString: String to search                                                 }
{ ASearchOption: Search Option                                              }
{     + opValue: Search in Value only                                       }
{     + opData: Search in Data only                                         }
{     + opValueData: Search in Value and Data                               }
{***************************************************************************}

procedure TRegistryEnhanced.Search(ASubKey,AString:string;ASearchOption:TSearchOption);
var
 TabIndex: array[0..MAX_DEPTH] of integer;
 tslKey: TStringList;
 i,counter,Depth,DeeperDepth: integer;
 down: boolean;
 tempKey, AKey: string;
begin
 FEntriesFound.clear;
 for i := 0 to MAX_DEPTH do TabIndex[i] := 0;   // initialisation

 FKey := CheckKey(FKey);
 if ASubKey <> '' then FKey := FKey + '\' + CheckKey(ASubKey);

 tempKey := FKey;
 tslKey := TStringList.create;
 counter := 1;
 Depth := 0;
 DeeperDepth := -1;
 down := true;

 { Get Root's ValueNames }
 RetrieveValueNames(AString,ASearchOption);

 { Go through keys }
 while (Counter > 0) do
  begin
   // Get the SubKeys
   GetSubKeys('',tslKey);
   if down then TabIndex[Depth] := tslKey.count;
   if Depth > DeeperDepth then DeeperDepth := Depth;     // Max Depth

   // Count the number of stuff reamining
   Counter := 0;
   for i := 0 to DeeperDepth do Counter := Counter + TabIndex[i];

   if (tslKey.count > 0) and (TabIndex[Depth] > 0) then
    begin
     // Change the key to a deeper key
     down := true;
     FKey := Fkey + '\' + tslKey[tslKey.count-TabIndex[Depth]];
     dec(TabIndex[Depth]);
     inc(Depth);
     if not RetrieveValueNames(AString,ASearchOption) then break;
    end
   else
    begin
     down := false;
     // Reset the number of SubKeys for the previous depth
     TabIndex[Depth] := 0;
     // Decrement the depth
     dec(Depth);
     // Change the key
     AKey := ExtractKeyPath(FKey);
     FKey := CheckKey(AKey);
    end;

   end;

   FKey := tempKey;
   tslKey.free;

end; {TRegistryEnhanced.Search}


{***************************************************************************}
{ PROCEDURE Register                                                        }
{***************************************************************************}

procedure Register;
begin
  RegisterComponents('Win95',[TRegistryEnhanced]);
end;

end.


